@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Björn Larsson
+# Copyright 2011 - 2013 Björn Larsson
 
 # This file is part of pytvdbapi.
 #
@@ -18,16 +18,17 @@
 # along with pytvdbapi.  If not, see <http://www.gnu.org/licenses/>.
 """
 A module containing classes and functions needed to be backwards compatible
-with python 2.5 and 2.6.
+with python 2.6.
 """
 
 import logging
 
 
 class NullHandler(logging.Handler):
-    """A replacement NullHandler for the one found in the standard library as
-     of version 2.7"""
-
+    """
+    A replacement NullHandler for the one found in the standard library as
+    of version 2.7
+    """
     def emit(self, record):
         """A do nothing emitter"""
         pass
