@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 - 2013 Björn Larsson
+# Copyright 2011 - 2014 Björn Larsson
 
 # This file is part of pytvdbapi.
 #
@@ -23,19 +23,21 @@ A module providing the default loader to use to load urls.
 
 import logging
 import os
+import zipfile
+from io import BytesIO
 
 import httplib2
 
 from pytvdbapi import error
-from pytvdbapi._compat import make_unicode
 
-#Module logger object
+
+# Module logger object
 logger = logging.getLogger(__name__)
 
 
 class Loader(object):
     """
-    A object for loading data from a provided url.
+    An object for loading data from a provided url.
     Uses httplib2 to do the heavy lifting.
     """
 
@@ -46,7 +48,7 @@ class Loader(object):
         """
         :param url: The URL to be loaded
         :param cache: Optional. Set if the cache should be ignored or not.
-        :return: The content of the url as bytes
+        :return: A file like object representing the loaded file content
         :raise: ConnectionError if the url could not be loaded
 
         """
@@ -67,5 +69,10 @@ class Loader(object):
             raise error.TVDBNotFoundError(u"Data not found")
         elif response.status not in [200, 304]:  # pragma: no cover
             raise error.ConnectionError(u"Bad status returned from server. {0}".format(response.status))
+
+        if response['content-type'] == "application/zip":
+            zip_file = zipfile.ZipFile(BytesIO(content))
+            filename = os.path.splitext(os.path.basename(url))[0]
+            return zip_file.open('{0}.xml'.format(filename))
         else:
-            return make_unicode(content)
+            return BytesIO(content)
