@@ -22,12 +22,14 @@ from __future__ import absolute_import, print_function
 import sys
 import unittest
 import datetime
+from io import StringIO
 
 import pytvdbapi
 from pytvdbapi import error
-from pytvdbapi.api import TVDB
+from pytvdbapi.api import TVDB, Episode, Show
 from pytvdbapi.xmlhelpers import generate_tree
 from pytvdbapi.tests import basetest
+from pytvdbapi._compat import make_unicode
 
 
 def _load_show(show):
@@ -62,7 +64,8 @@ class TestApi(basetest.pytvdbapiTest):
         """
         The tree generator should raise BadData error when passed bad xml data
         """
-        data = '<?xml version="1.0" encoding="UTF-8" ?>\n<data>'
+
+        data = StringIO(make_unicode('<?xml version="1.0" encoding="UTF-8" ?>\n<data>'))
 
         self.assertRaises(error.BadData, generate_tree, data)
 
@@ -103,7 +106,7 @@ class TestApi(basetest.pytvdbapiTest):
 
     def test_cache_dir(self):
         """It should be possible to specify a custom cache directory"""
-        #TODO: Implement this
+        # TODO: Implement this
 
     def test_version_format(self):
         """The package version string should be properly formatted"""
@@ -142,7 +145,7 @@ class TestSeason(unittest.TestCase):
         season1 = self.friends[1]
 
         for ep in season1:
-            self.assertEqual(type(ep), pytvdbapi.api.Episode)
+            self.assertEqual(type(ep), Episode)
 
     def test_season_sort_order(self):
         """The Episodes should be sorted on the episode number when iterating
@@ -292,14 +295,14 @@ class TestShow(unittest.TestCase):
         self.assertEqual(friends.SeriesName, "Friends")
         self.assertEqual(friends.id, 79168)
 
-        #This should not yet be loaded so should raise an error
+        # This should not yet be loaded so should raise an error
         self.assertRaises(
             error.TVDBAttributeError, friends.__getattr__, "Genre")
 
-        #Load in the rest of the attributes
+        # Load in the rest of the attributes
         friends.update()
 
-        #Now this data should be available
+        # Now this data should be available
         self.assertEqual(friends.Genre, ['Comedy'])
 
     def test_invalid_show_attribute(self):
@@ -373,10 +376,10 @@ class TestShow(unittest.TestCase):
     def test_invalid_index(self):
         """class should raise an exception if trying to use an invalid index"""
 
-        #Non integer index
+        # Non integer index
         self.assertRaises(error.TVDBValueError, self.friends.__getitem__, 'foo')
 
-        #Index out of range
+        # Index out of range
         self.assertRaises(error.TVDBIndexError, self.friends.__getitem__, 9999)
 
     def test_slice(self):
@@ -403,7 +406,7 @@ class TestShow(unittest.TestCase):
 
         for attr_name in dir(self.friends):
             attr = getattr(self.friends, attr_name)
-            if type(attr) not in (float, int, bool, datetime.date, pytvdbapi.api.TVDB):
+            if type(attr) not in (float, int, bool, type(None), datetime.date, pytvdbapi.api.TVDB):
                 if type(attr) in (list,):
                     for a in attr:
                         self.assertEqual(type(a), _type,
@@ -453,10 +456,10 @@ class TestEpisode(unittest.TestCase):
 
         ep = self.friends[1][1]
 
-        self.assertEquals(True, hasattr(ep, 'Rating'))
-        self.assertEquals(False, hasattr(ep, 'foo'))
+        self.assertEqual(True, hasattr(ep, 'Rating'))
+        self.assertEqual(False, hasattr(ep, 'foo'))
 
-        self.assertEquals(getattr(ep, 'foo', 'baar'), 'baar')
+        self.assertEqual(getattr(ep, 'foo', 'baar'), 'baar')
 
     def test_episode_pickle(self):
         """It should be possible to pickle and unpickle an episode"""
@@ -512,7 +515,7 @@ class TestSearch(unittest.TestCase):
         search = api.search("house", "en")
 
         for s in search:
-            self.assertEqual(type(s), pytvdbapi.api.Show)
+            self.assertEqual(type(s), Show)
 
     def test_search(self):
         """It should be possible to search for shows"""
@@ -580,7 +583,7 @@ class TestSearch(unittest.TestCase):
 
 
 class TestGetSeries(unittest.TestCase):
-    def test_get(self):
+    def test_get_series(self):
         """Provided the show id, you should be able to get the show object"""
         api = TVDB("B43FF87DE395DF56")
         show = api.get_series(79349, "en")
@@ -606,6 +609,37 @@ class TestGetSeries(unittest.TestCase):
         self.assertRaises(error.TVDBIdError, api.get_series, "foo", "en")
         self.assertRaises(error.TVDBIdError, api.get_series, "", "en")
 
+    def test_invalid_id_type(self):
+        """Function should raise Value error if the wrong type is provided"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBValueError, api.get_series, 79349, 'en', 'foo')
+
+    def test_imdb_id(self):
+        """It should be possible to use the imdb id to get the series"""
+        api = TVDB("B43FF87DE395DF56")
+
+        show = api.get_series(773262, 'en', 'imdb')
+        self.assertEqual(show.seriesid, 79349)  # Dexter
+
+        show = api.get_series('tt0773262', 'en', 'imdb')
+        self.assertEqual(show.seriesid, 79349)  # Dexter
+
+    def test_zap2it_id(self):
+        """It should be possible to load the show using the zap2it id"""
+        api = TVDB("B43FF87DE395DF56")
+
+        show = api.get_series('EP00859795', 'en', 'zap2it')
+        self.assertEqual(show.seriesid, 79349)  # Dexter
+
+        show = api.get_series(859795, 'en', 'zap2it')
+        self.assertEqual(show.seriesid, 79349)  # Dexter
+
+    def test_bad_id(self):
+        """Function should raise value error if a bad id is passed"""
+        api = TVDB("B43FF87DE395DF56")
+        self.assertRaises(error.TVDBValueError, api.get_series, api, 'en', 'foo')
+
 
 class TestGetEpisode(unittest.TestCase):
     def test_get_episode(self):
@@ -614,7 +648,7 @@ class TestGetEpisode(unittest.TestCase):
         get episode instance.
         """
         api = TVDB("B43FF87DE395DF56")
-        ep = api.get_episode(308834, "en")
+        ep = api.get_episode(308834, "en", "id")
 
         self.assertEqual(ep.id, 308834)
         self.assertEqual(ep.EpisodeName, 'Crocodile')
@@ -629,16 +663,294 @@ class TestGetEpisode(unittest.TestCase):
         self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "foo")
         self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "")
 
-    def test_invalid_id(self):
-        """
-        If the episode can not be found, a TVDBValueError should be raised.
-        """
+    def test_missing_id(self):
+        """It should be possible to call get_episode without passing id as positional argument"""
         api = TVDB("B43FF87DE395DF56")
 
-        self.assertRaises(error.TVDBIdError, api.get_episode, -1, "en")
-        self.assertRaises(error.TVDBIdError, api.get_episode, "foo", "en")
-        self.assertRaises(error.TVDBIdError, api.get_episode, "", "en")
+        ep = api.get_episode("en", "id", True, episodeid=308834)
 
+        self.assertEqual(ep.id, 308834)
+        self.assertEqual(ep.EpisodeName, 'Crocodile')
+
+    def test_episode_id_kwarg(self):
+        """It should be possible to load the episode passing the episode id as a kwarg"""
+        api = TVDB("B43FF87DE395DF56")
+
+        ep = api.get_episode(0, "en", "id", True, episodeid=308834)
+
+        self.assertEqual(ep.id, 308834)
+        self.assertEqual(ep.EpisodeName, 'Crocodile')
+
+    def test_dvd_order(self):
+        """it should be possible to get an episode using dvd sort ordering"""
+        api = TVDB("B43FF87DE395DF56")
+
+        ep = api.get_episode(0, "en", "dvd", True, seriesid=79349, seasonnumber=2, episodenumber=5)
+
+        self.assertEqual(ep.absolute_number, 17)
+
+    def test_default_order(self):
+        """It should be possible to get an episode using the default sort order"""
+        api = TVDB("B43FF87DE395DF56")
+
+        ep = api.get_episode(0, "en", "default", True, seriesid=79349, seasonnumber=2, episodenumber=5)
+
+        self.assertEqual(ep.absolute_number, 17)
+
+    def test_absolute_order(self):
+        """it should be possible to get an episode using absolute number"""
+        api = TVDB("B43FF87DE395DF56")
+
+        ep = api.get_episode(0, "en", "absolute", True, seriesid=81797, absolutenumber=62)
+        self.assertEqual(ep.EpisodeName, "The First Obstacle? Giant Whale Laboon Appears")
+
+    def test_invalid_method(self):
+        """Function should raise TVDBValueError if an invalid method is passed"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", -1)
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "foo")
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "")
+
+    def test_missing_arguments(self):
+        """Function should raise TVDBValueError if arguments for the method is missing"""
+        api = TVDB("B43FF87DE395DF56")
+
+        # self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "id")
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "default")
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "dvd")
+        self.assertRaises(error.TVDBValueError, api.get_episode, 308834, "en", "absolute")
+
+    def test_invalid_id(self):
+        """Function should raise bad data exception if an invalid id is passed"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.BadData, api.get_episode, "en", "id", True, episodeid=0, seriesid=81797)
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode, "en", "id", True, episodeid=-1, seriesid=81797)
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode, "en", "id", True, episodeid="foo", seriesid=81797)
+        self.assertRaises(error.BadData,
+                          api.get_episode, "en", "id", True, episodeid=99999999999, seriesid=81797)
+
+
+class TestGetEpisodeByAirDate(unittest.TestCase):
+    """Test the get_episode_by_air_date function"""
+
+    def test_non_date(self):
+        """Function should raise an exception if the air_date parameter is of the wrong type"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBValueError, api.get_episode_by_air_date, 79349, 'en', 'foo')
+        self.assertRaises(error.TVDBValueError, api.get_episode_by_air_date, 79349, 'en', '2001-12-21')
+        self.assertRaises(error.TVDBValueError, api.get_episode_by_air_date, 79349, 'en', 2)
+
+    def test_valid_show(self):
+        """Given correct values, the function should return a valid Episode instance"""
+        api = TVDB("B43FF87DE395DF56")
+
+        # Second episode of dexter
+        ep = api.get_episode_by_air_date(79349, 'en', datetime.date(2006, 10, 8))
+
+        self.assertEqual(ep.id, 308834)
+        self.assertEqual(ep.EpisodeName, 'Crocodile')
+
+    def test_no_episode_found(self):
+        """Function should raise an exception if no episode can be found"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode_by_air_date, 79349, 'en', datetime.date(2006, 10, 9))
+
+    def test_invalid_id(self):
+        """Function should raise an exception if an invalid id is used"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode_by_air_date, 'foo', 'en', datetime.date(2006, 10, 8))
+
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode_by_air_date, -1, 'en', datetime.date(2006, 10, 8))
+
+        self.assertRaises(error.TVDBNotFoundError,
+                          api.get_episode_by_air_date, 999999, 'en', datetime.date(2006, 10, 8))
+
+    def test_invalid_language(self):
+        """Function should raise an exception if an invalid language is passed"""
+        api = TVDB("B43FF87DE395DF56")
+
+        self.assertRaises(error.TVDBValueError,
+                          api.get_episode_by_air_date, 79349, 'foo', datetime.date(2006, 10, 8))
+
+        self.assertRaises(error.TVDBValueError,
+                          api.get_episode_by_air_date, 79349, 2, datetime.date(2006, 10, 8))
+
+        self.assertRaises(error.TVDBValueError,
+                          api.get_episode_by_air_date, 79349, 'english', datetime.date(2006, 10, 8))
+
+
+class TestFindOnShow(unittest.TestCase):
+    def setUp(self):
+        self.friends = _load_show('friends')
+        self.friends.update()
+
+    def test_bad_key_type(self):
+        """An exception should be raised if the key is not callable"""
+        self.assertRaises(error.TVDBTypeError, self.friends.find, "foo")
+
+        foo = 2
+        self.assertRaises(error.TVDBTypeError, self.friends.find, foo)
+
+    def test_filter_not_found(self):
+        """If no episode were found, an empty list should be returned"""
+        ep = self.friends.find(key=lambda _ep: getattr(_ep, 'ProductionCode', 0) == -1)
+        self.assertEqual(ep, None)
+
+    def test_key_function(self):
+        """The key function should be passed an episode instance"""
+        def _func(_ep):
+            self.assertEqual(type(_ep), Episode)
+        self.friends.find(key=_func)
+
+    def test_find(self):
+        """It should be possible to find an episode on a show"""
+        ep = self.friends.find(key=lambda _ep: getattr(_ep, 'ProductionCode', 0) == 457301)
+        self.assertEqual(ep.id, 303845)
+
+    def test_callable_function(self):
+        """It should be possible to use a callable function as key"""
+
+        class Key(object):
+            def __call__(self, _ep, *args, **kwargs):
+                return getattr(_ep, 'ProductionCode', 0) == 457301
+
+        ep = self.friends.find(key=Key())
+        self.assertEqual(ep.id, 303845)
+
+
+class TestFindOnSeason(unittest.TestCase):
+    def setUp(self):
+        self.friends = _load_show('friends')
+        self.friends.update()
+        self.season = self.friends[2]
+
+    def test_bad_key_type(self):
+        """If key is not callable, an exception should be raised"""
+        self.assertRaises(error.TVDBTypeError, self.season.find, 'foo')
+
+        foo = 2
+        self.assertRaises(error.TVDBTypeError, self.season.find, foo)
+
+    def test_not_found_find(self):
+        """If no object could be found, None should be returned"""
+        ep = self.season.find(key=lambda _ep: getattr(_ep, 'ProductionCode', 0) == -1)
+        self.assertEqual(ep, None)
+
+    def test_key_function(self):
+        """The key function should be passed an episode instance"""
+
+        def _func(_ep):
+            self.assertEqual(type(_ep), Episode)
+        self.season.find(key=_func)
+
+    def test_find(self):
+        """it should be possible to find an episode on the season instance"""
+        ep = self.season.find(key=lambda _ep: getattr(_ep, 'ProductionCode', 0) == 457301)
+        self.assertEqual(ep.id, 303845)
+
+    def test_callable_function(self):
+        """It should be possible to use a callable function as key"""
+
+        class Key(object):
+            def __call__(self, _ep, *args, **kwargs):
+                return getattr(_ep, 'ProductionCode', 0) == 457301
+        ep = self.season.find(key=Key())
+        self.assertEqual(ep.id, 303845)
+
+
+class TestFilterShow(unittest.TestCase):
+    def setUp(self):
+        self.friends = _load_show('friends')
+        self.friends.update()
+
+    def test_bad_key_type(self):
+        """An exception should be raised if the key is not callable"""
+        self.assertRaises(error.TVDBTypeError, self.friends.filter, "foo")
+
+        foo = 2
+        self.assertRaises(error.TVDBTypeError, self.friends.filter, foo)
+
+    def test_filter_not_found(self):
+        """If no episode were found, an empty list should be returned"""
+        ep = self.friends.filter(key=lambda _ep: getattr(_ep, "ProductionCode", 0) == -1)
+        self.assertNotEqual(ep, None)
+
+        self.assertEqual(len(ep), 0)
+        self.assertEqual(type(ep), list)
+
+    def test_key_function(self):
+        """The key function should be passed an episode instance"""
+
+        def _func(_ep):
+            self.assertEqual(type(_ep), Episode)
+        self.friends.filter(key=_func)
+
+    def test_filter(self):
+        """It should be possible to filter the episodes on a season"""
+        eps = self.friends.filter(key=lambda _ep: "Lauren Tom" in getattr(_ep, "GuestStars", list()))
+        self.assertEqual(len(eps), 7)
+
+    def test_callable_function(self):
+        """It should be possible to use a callable function as key"""
+
+        class Key(object):
+            def __call__(self, ep, *args, **kwargs):
+                return "Lauren Tom" in getattr(ep, "GuestStars", list())
+        eps = self.friends.filter(key=Key())
+        self.assertEqual(len(eps), 7)
+
+
+class TestFilterSeason(unittest.TestCase):
+    def setUp(self):
+        self.friends = _load_show('friends')
+        self.friends.update()
+        self.season = self.friends[2]
+
+    def test_bad_key_type(self):
+        """An exception should be raised if the key is not callable"""
+        self.assertRaises(error.TVDBTypeError, self.season.filter, "foo")
+
+        foo = 2
+        self.assertRaises(error.TVDBTypeError, self.season.filter, foo)
+
+    def test_filter_not_found(self):
+        """If no episode were found, an empty list should be returned"""
+        ep = self.season.filter(key=lambda _ep: getattr(_ep, "ProductionCode", 0) == -1)
+        self.assertNotEqual(ep, None)
+
+        self.assertEqual(len(ep), 0)
+        self.assertEqual(type(ep), list)
+
+    def test_key_function(self):
+        """The key function should be passed an episode instance"""
+
+        def _func(_ep):
+            self.assertEqual(type(_ep), Episode)
+        self.season.filter(key=_func)
+
+    def test_filter(self):
+        """It should be possible to filter the episodes on a season"""
+        eps = self.season.filter(key=lambda _ep: "Lauren Tom" in getattr(_ep, "GuestStars", list()))
+        self.assertEqual(len(eps), 6)
+
+    def test_callable_function(self):
+        """It should be possible to use a callable function as key"""
+
+        class Key(object):
+            def __call__(self, ep, *args, **kwargs):
+                return "Lauren Tom" in getattr(ep, "GuestStars", list())
+        eps = self.season.filter(key=Key())
+        self.assertEqual(len(eps), 6)
 
 if __name__ == "__main__":
     sys.exit(unittest.main())
