@@ -17,22 +17,23 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with pytvdbapi.  If not, see <http://www.gnu.org/licenses/>.
 
-#Imports for a more Py3K functionality
+# Imports for a more Py3K functionality
 from __future__ import absolute_import, print_function
 
 import sys
 import unittest
 import os
+from io import BytesIO
 
 from pytvdbapi import xmlhelpers, mirror, error
-from pytvdbapi.tests import utils, basetest
+from pytvdbapi.tests import basetest
 
 
 class TestMirror(basetest.pytvdbapiTest):
     def setUp(self):
         super(TestMirror, self).setUp()
 
-        data = utils.file_loader(os.path.join(self.path, "mirrors.xml"))
+        data = BytesIO(open(os.path.join(self.path, "mirrors.xml"), 'rb').read())
         self.mirrors = mirror.MirrorList(xmlhelpers.generate_tree(data))
 
     def tearDown(self):
