@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Björn Larsson
+# Copyright 2011 - 2013 Björn Larsson
 
 # This file is part of pytvdbapi.
 #
@@ -19,13 +19,23 @@
 
 __all__ = ['file_loader']
 
+# pylint: disable W0622
+try:
+    from io import open  # For Py 2.6 - 2.7
+except ImportError:
+    pass
+# pylint: enable W0622
 
-def file_loader(file):
+
+def file_loader(_file):
+    """
+    :param _file: Path to load
+    """
     try:
-        handle = open(file)
+        handle = open(_file, mode='rt', encoding='utf-8')
         data = handle.read()
     except IOError:
-        print("Unable to open {0}".format(file))
+        print("Unable to open {0}".format(_file))
         data = ""
     finally:
         handle.close()
