@@ -25,9 +25,11 @@ A module for utility functionality.
 
 from functools import wraps
 from collections import MutableMapping
-from pytvdbapi._compat import make_unicode
 
-__all__ = ['unicode_arguments', 'merge', 'TransformedDictionary', 'InsensitiveDictionary']
+from pytvdbapi._compat import make_unicode, int_types
+
+
+__all__ = ['unicode_arguments', 'deprecate_episode_id', 'TransformedDictionary', 'InsensitiveDictionary']
 
 
 def unicode_arguments(func):
@@ -47,24 +49,27 @@ def unicode_arguments(func):
     return __wrapper__
 
 
-def merge(dict1, dict2, decision=lambda x, y: y):
+def deprecate_episode_id(func):
     """
-    :param dict1: First dictionary to merge
-    :param dict2: Second dictionary to merge
-    :param decision: A callable taking two values v1, v2 returning the value
-        to keep. The default is to keep values in dict2.
-    :return: A new dictionary with the merged result
+    Utility function to help deprecating the episode_id parameter without loosing backward compatibility.
 
-    Merging two dictionaries together using *decision* to determine what
-    values will be used.
+    :param func: The function to wrap
+    :type func: A function object
     """
-    result = dict1
-    for key, value in list(dict2.items()):
-        if key in result:
-            result[key] = decision(result[key], value)
-        else:
-            result[key] = value
-    return result
+    @wraps(func)
+    def __wrapper__(*args, **kwargs):
+        args = list(args)
+
+        # Check if the first argument is an int, and assume that it was the episode id
+        if isinstance(args[1], int_types):
+            episodeid = args[1]
+            if 'episodeid' not in kwargs:
+                kwargs['episodeid'] = episodeid
+
+            del args[1]
+
+        return func(*args, **kwargs)
+    return __wrapper__
 
 
 class TransformedDictionary(MutableMapping, object):
