@@ -24,26 +24,42 @@ A helper module for parsing XML data.
 import datetime
 import logging
 import re
-import xml.etree.ElementTree as eTree
 
+# pylint: disable=E0611
 try:
-    from xml.etree.ElementTree import ParseError  # pylint: disable=E0611
+    import xml.etree.cElementTree as eTree
+    from xml.etree.cElementTree import ParseError
 except ImportError:
-    # For Python 2.6
-    from xml.parsers.expat import ExpatError as ParseError
+    import xml.etree.ElementTree as eTree
+
+    try:
+        from xml.etree.ElementTree import ParseError
+    except ImportError:
+        # Python 2.6
+        from xml.parsers.expat import ExpatError as ParseError
 
 from pytvdbapi import error
-from pytvdbapi._compat import make_bytes, make_unicode
+from pytvdbapi._compat import make_unicode
 
-__all__ = ['generate_tree', 'parse_xml']
+__all__ = ['has_element', 'generate_tree', 'parse_xml']
 
-#Module level logger object
+# Module level logger object
 logger = logging.getLogger(__name__)
 
 
+def has_element(etree, element):
+    """
+    :param etree: the element tree to check
+    :param element: the element to check for
+
+    Check if the elementree contains the provided element name
+    """
+    return etree.find(element) is not None
+
+
 def generate_tree(xml_data):
     """
-    :param xml_data: The string xml data to generate the tree from
+    :param xml_data: A file like object containing the xml data
 
     :return: A new ElementTree
     :raise: :class:`pytvdbapi.error.BadData`
@@ -51,7 +67,7 @@ def generate_tree(xml_data):
     Converts the xml data into an element tree
     """
     try:
-        return eTree.fromstring(make_bytes(xml_data, 'utf-8'))
+        return eTree.parse(xml_data)
     except ParseError:
         raise error.BadData(u"Bad XML data received")
 
